/*
scripsi
copyright 2010 jezra lickter http://www.jezra.net

scripsi is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

scripsi is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with scripsi.  If not, see <http://www.gnu.org/licenses/>.

*/

using Gtk;
using Gdk;

private class TextEditor:Gtk.VBox{
  public TextView text_view;
  private TextBuffer text_buffer;
  private ScrolledWindow scroll;
  private Color focused_color;
  private bool is_editable;
  private string editing_file;
  construct{
    Color.parse("#b50",out focused_color);
  }
  
  public TextEditor(bool display_save=false) {
    is_editable = display_save;
    //start building the ui
    var toolbar = new Toolbar ();
    var open_button = new ToolButton.from_stock (STOCK_OPEN);
    open_button.clicked.connect (on_open_clicked);
    toolbar.add(open_button);
    //make the text buffer and view
    text_buffer = new TextBuffer(null);
    text_view = new TextView.with_buffer( text_buffer );
    text_view.set_border_width(2);
    text_view.set_wrap_mode(WrapMode.WORD_CHAR);
    //we need to know when the text buffer changes
    text_buffer.changed.connect( text_buffer_changed);
    
    //are we displaying the "save button"?
    if (display_save) {
      var save_button = new ToolButton.from_stock (STOCK_SAVE);
      toolbar.add(save_button);
      save_button.clicked.connect(save);
    }else{
      //no save means no editing
      text_view.set_editable(false);
    }
    
    scroll = new ScrolledWindow (null, null);
    scroll.set_policy (PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
    scroll.add (this.text_view);
    pack_start (toolbar, false, true, 0);
    pack_start (scroll, true, true, 0);
    
    //connect some signals to handle focus of the textview
    text_view.focus_in_event.connect( ()=>{ 
        this.focus_in();
        return false;
    } );
    text_view.focus_out_event.connect( ()=>{ 
        this.focus_out();
        return false;
    } );
  }
  
  //what happens when the text buffer changes?
  private void text_buffer_changed() {
    //TextIter iter = new TextIter();
    TextIter iter = TextIter();
    //make sure that the associated text_view has the cursor visible
    int x = text_view.virtual_cursor_x;
    int y = text_view.virtual_cursor_y;
    text_view.get_iter_at_location(out iter,x,y);
    text_view.scroll_to_iter(iter,0,false,0,0);
  }
  
  
  private void focus_in() {
    text_view.modify_bg(StateType.NORMAL,focused_color );
    //scroll.set_shadow_type(ShadowType.ETCHED_OUT);
  }
  
  private void focus_out() {
    text_view.modify_bg(StateType.NORMAL,null );
    //scroll.set_shadow_type(ShadowType.NONE);
  }
  
  public void on_open_clicked () {
    var file = choose_file ( _("Open File"), FileChooserAction.OPEN );
    if (file!=null) {
      open_file (file);
    }
  }
  
  public void save() {
    string text = text_view.buffer.text;
    //is there a set editing file?
    if (editing_file!=null) {
      try {
        FileUtils.set_contents(editing_file, text);
      }catch(FileError e) {
        stderr.printf (_("Error: %s\n"), e.message);
      }
    }else{
      //since there is no editing file, we need to save as..
      save_as();
    }
  }
  
  private void save_as() {
    var save_file = choose_file( _("Save As...") , FileChooserAction.SAVE );
    if(save_file!=null) {
      editing_file=save_file;
      save();
    }
  }

  private void open_file (string filename) {
    try {
      string text;
      FileUtils.get_contents (filename, out text, null);
      this.text_view.buffer.text = text;
      editing_file = filename;
    } catch (Error e) {
      stderr.printf (_("Error: %s\n"), e.message);
    }
  }
  
  private string choose_file(string dialog_title, FileChooserAction action ) {
    string file = null;
    var file_chooser = new FileChooserDialog (dialog_title, null,
          action,
          STOCK_CANCEL, ResponseType.CANCEL,
          STOCK_OPEN, ResponseType.ACCEPT, null);
          
    if (file_chooser.run () == ResponseType.ACCEPT) {
      file = file_chooser.get_filename ();
    }
    file_chooser.destroy ();
    return file;
  }
}


public class MainWindow:Gtk.Window {
  public signal void quit_activated();
  private TextEditor texteditor1;
  private TextEditor texteditor2;
  
  construct{
    //what happens during construct? nothing?
  }
  public MainWindow(){
    //main window should be full screen
    maximize();
    //create an accel group
    AccelGroup accelGroup = new AccelGroup();
    //add teh accelgroup to the window
    add_accel_group(accelGroup);
    //create the text editors
    texteditor1 = new TextEditor();
    texteditor2 = new TextEditor(true);
    //put the editors in a hpane
    var editorsHPaned = new HPaned();
    editorsHPaned.pack1(texteditor1,true,true);
    editorsHPaned.pack2(texteditor2,true,true);
    
    //create the menu
    MenuBar mainMenu = new MenuBar();
    Menu fileMenu = new Menu();
    //make the top level item
    MenuItem fileMenuLauncher = new MenuItem.with_mnemonic( _("_File") );
    //make the menu a subment of the launcher
    fileMenuLauncher.set_submenu(fileMenu);
    //make the "open" items
    var openOriginalItem = new MenuItem.with_mnemonic( _("_Open Original Source") );
    var openTranslationItem = new MenuItem.with_mnemonic( _("O_pen Translation") );
    var saveItem = new MenuItem.with_mnemonic( _("_Save") );
    var quitItem = new MenuItem.with_mnemonic( _("_Quit") );
    //add accelerator shit
    openOriginalItem.add_accelerator("activate", accelGroup, 'o',
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);
    openTranslationItem.add_accelerator("activate", accelGroup, 'p',
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);
    quitItem.add_accelerator("activate", accelGroup, 'q',
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);
    saveItem.add_accelerator("activate", accelGroup, 's',
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);
    //add activation actions
    openOriginalItem.activate.connect( texteditor1.on_open_clicked );
    openTranslationItem.activate.connect( texteditor2.on_open_clicked );
    quitItem.activate.connect( quit );
    saveItem.activate.connect(texteditor2.save);
    
    //append the items to the menu
    fileMenu.append(openOriginalItem);
    fileMenu.append(openTranslationItem);
    fileMenu.append(saveItem);
    fileMenu.append(quitItem);
    mainMenu.insert(fileMenuLauncher,0);
    
    //make the "select" menu
    Menu selectMenu = new Menu();
    //make the top level item
    MenuItem selectMenuLauncher = new MenuItem.with_mnemonic( _("_Select") );
    //make the menu a subment of the launcher
    selectMenuLauncher.set_submenu(selectMenu);
    //make the "open" items
    var originalItem = new MenuItem.with_mnemonic( _("_Original Source") );
    var translationItem = new MenuItem.with_mnemonic( _("_Translation") );
    //add accelerator shit ( can you tell I did some copy/paste?
    originalItem.add_accelerator("activate", accelGroup, 65361,
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);
    translationItem.add_accelerator("activate", accelGroup, 65363,
                    Gdk.ModifierType.CONTROL_MASK, 
                    Gtk.AccelFlags.VISIBLE);

    //add activation actions
    originalItem.activate.connect( ()=>{
        select_texteditor( ref texteditor1 );
    });
    translationItem.activate.connect( ()=>{
        select_texteditor( ref texteditor2 );
    });
    
    //append the items to the menu
    selectMenu.append(originalItem);
    selectMenu.append(translationItem);
    mainMenu.append(selectMenuLauncher);
      
    //create a vbox to hold stuff
    var vBox = new VBox(false,5);
    vBox.pack_start(mainMenu,false,false,0);
    vBox.pack_start(editorsHPaned);
    add(vBox);
  }
  
  //select the text editor by it reference
  private void select_texteditor( ref TextEditor te ){
    set_focus(te.text_view);
  }
  
  private void quit(){
    //emit the destroy event
    quit_activated();
  }
}
